package com.jz.common.utils;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.gson.Gson;
import com.jz.common.utils.web.HttpClientUtils;
import com.jz.common.utils.web.SessionUtils;
import com.jz.dmp.modules.controller.DataIntegration.bean.flow.FlowExecution;
import com.jz.dmp.modules.controller.DataIntegration.bean.flow.FlowPro;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.FileSystemResource;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestTemplate;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * azkaban ajax api 工具类
 */
public class AzkabanApiUtils2 {


    private static final Logger LOGGER = LoggerFactory.getLogger(AzkabanApiUtils2.class);

    private String azkabanServerUrl;
    private String userName;
    private String password;

    public AzkabanApiUtils2(String azkabanServerUrl, String userName, String password) {
        this(azkabanServerUrl);
        this.userName = userName;
        this.password = password;
    }

    public AzkabanApiUtils2(String azkabanServerUrl) {
        this.azkabanServerUrl = azkabanServerUrl;
        this.userName = "admin";
        this.password = "admin";
    }

    /**
     * 构造RestTemplate实例，发起ajax请求
     *
     * @return
     */
    public RestTemplate bulidRestTemplate() {
        SimpleClientHttpRequestFactory requestFactory = new SimpleClientHttpRequestFactory();
        requestFactory.setConnectTimeout(900000);
        requestFactory.setReadTimeout(900000);
        RestTemplate rest = new RestTemplate(requestFactory);
        return rest;
    }

    /**
     * 调用登陆接口,得到sessionId
     *
     * @return String sessionId 会话id
     * @throws Exception
     */
    @SuppressWarnings("unchecked")
    public String login() {

        /*HttpHeaders hs = new HttpHeaders();
        hs.add("Content-Type", "application/x-www-form-urlencoded; charset=utf-8");
        hs.add("X-Requested-With", "XMLHttpRequest");
        LinkedMultiValueMap<String, String> linkedMultiValueMap = new LinkedMultiValueMap<String, String>();
        linkedMultiValueMap.add("action", "login");
        linkedMultiValueMap.add("username", userName);
        linkedMultiValueMap.add("password", password);
        HttpEntity<MultiValueMap<String, String>> httpEntity = new HttpEntity<>(linkedMultiValueMap, hs);
        Map<String, String> postForObject = null;
        try {
        	 postForObject = bulidRestTemplate().postForObject(azkabanServerUrl, httpEntity, Map.class);
		} catch (Exception e) {
			e.printStackTrace();
			LOGGER.error(azkabanServerUrl+"-----"+linkedMultiValueMap+"调用登录接口异常");
			throw new RuntimeException("登录异常");
		}
        String sessionId = postForObject.get("session.id");
        if (StringUtils.isEmpty(sessionId)) {
        	LOGGER.error(azkabanServerUrl+"-----"+linkedMultiValueMap+" sessionId 为空");
			throw new RuntimeException("登陆失败");
		}*/
        String sessionId = SessionUtils.getSession().getId();  //"dcfc608c-c58a-45b7-adc7-9902b652496e";
    	//String sessionId = "f70d53fa-55da-4688-8d00-64350e4fb8ea";
    	
        System.err.println("----sessionId="+sessionId);
        return sessionId; //SessionUtils.getSession().getId();
    }

    /**
     * 创建azkaban项目名
     *
     * @param projectName azkaban项目名  对应 dmp项目下的工作流任务名
     * @param projectDesc azkaban项目描述
     * @return 创建成功返回项目名, 创建失败   返回
     */
    @SuppressWarnings("unchecked")
    public String createAzkabanProject(String sessionId, String projectName, String projectDesc) {
        String projectManageUrl = azkabanServerUrl + "/manager?token=" + sessionId;

        if (StringUtils.isEmpty(sessionId)) {
            throw new RuntimeException("还未登录");
        }

        if (StringUtils.isEmpty(projectName)) {
            throw new RuntimeException("项目名不能为空");
        }

        if (StringUtils.isEmpty(projectDesc)) {
            throw new RuntimeException("项目描述不能为空");
        }

        HttpHeaders hs = new HttpHeaders();
        hs.add("Content-Type", "application/x-www-form-urlencoded; charset=utf-8");
        hs.add("X-Requested-With", "XMLHttpRequest");
        LinkedMultiValueMap<String, String> linkedMultiValueMap = new LinkedMultiValueMap<String, String>();
        linkedMultiValueMap.add("session.id", sessionId);
        linkedMultiValueMap.add("action", "create");
        linkedMultiValueMap.add("name", projectName);
        linkedMultiValueMap.add("description", projectDesc);
        HttpEntity<MultiValueMap<String, String>> httpEntity = new HttpEntity<>(linkedMultiValueMap, hs);

        String result = null;
        try {
            result = bulidRestTemplate().postForObject(projectManageUrl, httpEntity, String.class);
        } catch (Exception e) {
            LOGGER.error(projectManageUrl + "----" + linkedMultiValueMap + "---调用创建项目失败");
            e.printStackTrace();
            throw new RuntimeException("创建项目失败");
        }
        System.err.println("createAzkabanProject   result--"+result);
        Map<String, String> postForObject = JSON.parseObject(result, Map.class);

        String status = postForObject.get("status");
        if (StringUtils.isEmpty(status)) {
            throw new RuntimeException("创建项目失败");
        } else {
            String message = postForObject.get("message");
            if ("error".equals(status)) {
                if ("Project already exists.".equals(message) || message.contains("already exists in db")) {

                } else {
                    throw new RuntimeException("创建项目失败");
                }
            }
        }
        LOGGER.info("创建项目 " + projectName + "成功");
        return projectName;
    }

    /**
     * 先创建项目  在上传zip
     *
     * @param sessionId     登录的会话id
     * @param projectName   项目名
     * @param uploadZipPath 上传的zip包全路径
     * @return
     */
    @SuppressWarnings("unchecked")
    public Map<String, Object> uploadZip(String sessionId, String projectName, String uploadZipPath) {
        String projectManageUrl = azkabanServerUrl + "/manager?token=" + sessionId;
        //上传zip
        FileSystemResource resource = new FileSystemResource(new File(uploadZipPath));
        if (!resource.exists()) {
            throw new RuntimeException("上传失败," + uploadZipPath + "不存在");
        }
        LinkedMultiValueMap<String, Object> linkedMultiValueMap = new LinkedMultiValueMap<String, Object>();
        linkedMultiValueMap.add("session.id", sessionId);
        linkedMultiValueMap.add("ajax", "upload");
        linkedMultiValueMap.add("project", projectName);
        linkedMultiValueMap.add("file", resource);
        Map<String, Object> postForObject = bulidRestTemplate().postForObject(projectManageUrl, linkedMultiValueMap, Map.class);
        return postForObject;
    }

    /**
     * 上传项目
     *
     * @param projectName   项目名称
     * @param uploadZipPath 本地项目压缩包路径
     * @return
     */
    public Map<String, Object> uploadZip(String projectName, String uploadZipPath) {
        String sessionId = login();
        return uploadZip(sessionId, projectName, uploadZipPath);
    }

    /**
     * 定时调度一个工作流
     *
     * @param sessionId   会话id
     * @param projectName 项目名
     * @param flowName    azkaban工作流名称
     * @param flowPro
     * @return 调用成功后，返回调度id
     */
    @SuppressWarnings("unchecked")
    public Integer scheduleFlowCron(String sessionId, String projectName, String flowName, FlowPro flowPro) {
        if (StringUtils.isEmpty(sessionId)) {
            throw new RuntimeException("还未登陆");
        }

        if (StringUtils.isEmpty(projectName)) {
            throw new RuntimeException("调度流程项目名不能为空");
        }

        if (StringUtils.isEmpty(flowName)) {
            throw new RuntimeException("调度流程  流程名不能为空");
        }

        if (StringUtils.isEmpty(flowPro.getScheduleSetting())) {
            throw new RuntimeException("调度流程  调度cron表达式不能为空");
        }

        String scheduleUrl = azkabanServerUrl + "/schedule?token=" + sessionId;
        LinkedMultiValueMap<String, Object> linkedMultiValueMap = new LinkedMultiValueMap<String, Object>();
        linkedMultiValueMap.add("session.id", sessionId);
        linkedMultiValueMap.add("ajax", "scheduleCronFlow");
        linkedMultiValueMap.add("projectName", projectName);
        linkedMultiValueMap.add("flow", flowName);
        linkedMultiValueMap.add("cronExpression", flowPro.getScheduleSetting());

        /*if (!StringUtils.isEmpty(workFlowTask.getSuccessEmails())) {
        	 linkedMultiValueMap.add("successEmails", workFlowTask.getSuccessEmails());
             //linkedMultiValueMap.add("successEmailsOverride", workFlowTask.isSuccessEmailsOverride());
		}

        if (!StringUtils.isEmpty(workFlowTask.getFailureEmails())) {
        	 linkedMultiValueMap.add("failureEmails", workFlowTask.getFailureEmails());
             //linkedMultiValueMap.add("failureEmailsOverride", workFlowTask.isFailureEmailsOverride());
		}*/

        Map<String, Object> postForObject = null;
        try {
            postForObject = bulidRestTemplate().postForObject(scheduleUrl, linkedMultiValueMap, Map.class);
        } catch (Exception e) {
            LOGGER.error(scheduleUrl + "----" + linkedMultiValueMap + "----调用任务调度接口异常");
            e.printStackTrace();
            throw new RuntimeException("定时调度接口异常");
        }

        if (postForObject.containsKey("error")){
            throw new RuntimeException(postForObject.get("error").toString());
        }
        String status = (String) postForObject.get("status");
        if ("error".equals(status)) {
            String message = (String) postForObject.get("message");
            throw new RuntimeException(message);
        }
        LOGGER.info("调度projectName=" + projectName + ",flowName=" + flowName + ",cronExpression=" + flowPro.getScheduleSetting() + "  成功");
        return (Integer) postForObject.get("scheduleId");
    }

    /**
     * 创建一个新的任务 ,包含  登录  创建项目  上传zip包  并定时调度
     *
     * @param projectName   项目名
     * @param projectDesc   项目描述
     * @param uploadZipPath 上传zip包的全路径
     */
    public boolean loginCreateProjectuploadZipAndSchedule(String projectName, String projectDesc, String uploadZipPath, FlowPro flowPro) {
        //登录
        String sessionId = login();
        //创建项目
        createAzkabanProject(sessionId, projectName, projectDesc);

            Map<String, Object> map = uploadZip(sessionId, projectName, uploadZipPath);
            if (map.containsKey("error")) {
                return false;
            }
            if (!StringUtils.isEmpty(flowPro.getScheduleSetting())) {
                String flowName = flowPro.getFlowName();
                //定时调度 设置不为空  直接调用接口进行调度
                scheduleFlowCron(sessionId, projectName, flowName, flowPro);
            }

        return true;
    }

    /**
     * 删除工作流
     *
     * @param project     azkaban项目名
     * @param projectName 任务名
     * @return
     */
    public Map<String, Object> deleteAzkabanFlow(String project, String projectName) {
        String sessionId = login();

        if (StringUtils.isEmpty(sessionId)) {
            throw new RuntimeException("还未登录");
        }
        if (StringUtils.isEmpty(project)) {
            throw new RuntimeException("项目名不能为空");
        }
        if (StringUtils.isEmpty(projectName)) {
            throw new RuntimeException("任务名不能为空");
        }

        LinkedMultiValueMap<String, String> linkedMultiValueMap = new LinkedMultiValueMap();
        String accessToken = "?token=" + sessionId;
        Map<String, Object> resp;
        String cleanUrl = "";
        String projectId;
        int i = 0;
        try {
            cleanUrl = azkabanServerUrl + "/manager" + accessToken;
            cleanUrl += "&ajax=fetchprojectflows&session.id=" + sessionId + "&project=" + project;
            LOGGER.info("{}.获取一个project的任务流-查询对应项目的信息及其任务流", ++i);
            resp = bulidRestTemplate().getForObject(cleanUrl, Map.class);
            LOGGER.info("url:{}, response:{}", cleanUrl, resp);

            if (null == resp || !resp.containsKey("projectId")) {
                throw new RuntimeException(projectName + "项目不存在！");
            }

            projectId = resp.get("projectId").toString();

            cleanUrl = azkabanServerUrl + "/schedule" + accessToken;
            cleanUrl += "&ajax=fetchSchedule&session.id=" + sessionId + "&projectId=" + projectId + "&flowId=" + projectName;
            LOGGER.info("{}.获取指定Project,flow下的数据流的调度-查询对应工作流的调度信息", ++i);
            resp = bulidRestTemplate().getForObject(cleanUrl, Map.class);
            LOGGER.info("url:{}, response:{}", cleanUrl, resp);
            String scheduleId = String.valueOf(EasyJsonUtils.getKeyJsonQuery(resp, "schedule.scheduleId"));

            if (!"null".equals(scheduleId)) {
                cleanUrl = azkabanServerUrl + "/schedule" + accessToken;
                linkedMultiValueMap.clear();
                linkedMultiValueMap.add("action", "removeSched");
                linkedMultiValueMap.add("session.id", sessionId);
                linkedMultiValueMap.add("scheduleId", scheduleId);
                LOGGER.info("{}.取消工作流的调度", ++i);
                resp = bulidRestTemplate().postForObject(cleanUrl, linkedMultiValueMap, Map.class);
                LOGGER.info("url:{}, param:{}, response:{}", cleanUrl, new Gson().toJson(linkedMultiValueMap), resp);
                if (null == resp && !resp.containsKey("status") || "error".equals(resp.get("status").toString()))
                    throw new RuntimeException(projectName + "取消工作流调度出错！");
            }

            cleanUrl = azkabanServerUrl + "/executor" + accessToken;
            cleanUrl += "&ajax=getRunning&session.id=" + sessionId + "&project=" + project + "&flow=" + projectName;
            LOGGER.info("{}.获取任务流正在执行的Execution-查询对应工作流的信息", ++i);
            resp = bulidRestTemplate().getForObject(cleanUrl, Map.class);
            LOGGER.info("url:{}, response:{}", cleanUrl, resp);

            if (null != resp && resp.containsKey("execIds")) {
                //List ids = JSONUtils.toObject(List.class, resp.get("execIds").toString());
                List ids = JSON.parseArray(resp.get("execIds").toString()).toJavaList(String.class);
                for (int j = 0; j < ids.size(); j++) {
                    cleanUrl = azkabanServerUrl + "/executor" + accessToken;
                    cleanUrl += "&ajax=cancelFlow&session.id=" + sessionId + "&execid=" + ids.get(j).toString();
                    LOGGER.info("{}.{}.取消流程执行-停止对应调度工作流", i, j);
                    resp = bulidRestTemplate().getForObject(cleanUrl, Map.class);
                    LOGGER.info("url:{}, response:{}", cleanUrl, resp);
                }
            }

            cleanUrl = azkabanServerUrl + "/workflow" + accessToken;
            linkedMultiValueMap.clear();
            linkedMultiValueMap.add("projectName", projectName);
            linkedMultiValueMap.add("type", "cleanFlow");
            linkedMultiValueMap.add("project", project);
            LOGGER.info("{}.清理azkaban相关表数据及缓存", ++i);
            resp = bulidRestTemplate().postForObject(cleanUrl, linkedMultiValueMap, Map.class);
            LOGGER.info("url:{}, param:{}, response:{}", cleanUrl, new Gson().toJson(linkedMultiValueMap), resp);
        } catch (Exception e) {
            LOGGER.error("工作流清理失败，cur_url-{}-param-{}", cleanUrl, new Gson().toJson(linkedMultiValueMap), e);
            throw new RuntimeException("工作流清理失败：" + e.getMessage());
        }
        if (resp.containsKey("error")) {
            throw new RuntimeException(resp.get("error").toString());
        }
        LOGGER.info("清理 {} 项目下的 {} 工作流成功", project, projectName);
        return resp;
    }

    /**
     * 检查项目下的流是否存在
     *
     * @param projectName
     * @param flowName
     * @return
     */
    public boolean checkFlowExists(String projectName, String flowName) {
        String accessToken = "?token=" + login();
        String url = azkabanServerUrl + "/manager" + accessToken + "&ajax=fetchflowgraph&project=" + projectName + "&flow=" + flowName;
        Map resultMap = bulidRestTemplate().getForObject(url, Map.class);
        if (resultMap.containsKey("error")) {
            return false;
        }
        return true;
    }
    
    /**
     * 创建一个新的任务 ,包含  登录  创建项目  上传zip包  并立即执行
     *
     * @param projectName   项目名
     * @param projectDesc   项目描述
     * @param uploadZipPath 上传zip包的全路径
     */
    public boolean loginCreateProjectuploadZipAndExecute(String projectName, String projectDesc, String uploadZipPath, String flowName) {
        //登录
        String sessionId = login();
    	
        //创建项目
        createAzkabanProject(sessionId, projectName, projectDesc);

            Map<String, Object> map = uploadZip(sessionId, projectName, uploadZipPath);
            if (map.containsKey("error")) {
                return false;
            }
            //立即执行
            executeFlow(sessionId, projectName, flowName);

        return true;
    }
    
    /**
     * 调度一个工作流(one time)
     *
     * @param sessionId   会话id
     * @param projectName 项目名
     * @param flowName    azkaban工作流名称
     * @return 调用成功后，返回execid
     */
    @SuppressWarnings("unchecked")
    public Integer executeFlow(String sessionId, String projectName, String flowName) {
        if (StringUtils.isEmpty(sessionId)) {
            throw new RuntimeException("还未登陆");
        }

        if (StringUtils.isEmpty(projectName)) {
            throw new RuntimeException("调度流程项目名不能为空");
        }

        if (StringUtils.isEmpty(flowName)) {
            throw new RuntimeException("调度流程  流程名不能为空");
        }

        String executeFlowUrl = azkabanServerUrl + "/executor?token=" + sessionId;
        LinkedMultiValueMap<String, Object> linkedMultiValueMap = new LinkedMultiValueMap<String, Object>();
        linkedMultiValueMap.add("session.id", sessionId);
        linkedMultiValueMap.add("ajax", "executeFlow");
        linkedMultiValueMap.add("project", projectName);
        linkedMultiValueMap.add("flow", flowName);

        Map<String, Object> postForObject = null;
        try {
            postForObject = bulidRestTemplate().postForObject(executeFlowUrl, linkedMultiValueMap, Map.class);
        } catch (Exception e) {
            LOGGER.error(executeFlowUrl + "----" + linkedMultiValueMap + "----调用任务调度接口异常");
            e.printStackTrace();
            throw new RuntimeException("调度接口异常");
        }

        if (postForObject.containsKey("error")){
            throw new RuntimeException(postForObject.get("error").toString());
        }
        String status = (String) postForObject.get("status");
        if ("error".equals(status)) {
            String message = (String) postForObject.get("message");
            throw new RuntimeException(message);
        }
        LOGGER.info("调度projectName=" + projectName + ",flowName=" + flowName + "one time 成功");
        return (Integer) postForObject.get("execid");
    }
    
    /**
     * 停止一个工作流调度
     *
     * @param sessionId   会话id
     * @param projectName 项目名
     * @param flowName    azkaban工作流名称
     * @return 
     */
    @SuppressWarnings("unchecked")
    public String stopFlow(String projectName, String flowName) {

        if (StringUtils.isEmpty(projectName)) {
            throw new RuntimeException("调度流程项目名不能为空");
        }

        if (StringUtils.isEmpty(flowName)) {
            throw new RuntimeException("调度流程  流程名不能为空");
        }
        //登录
        String sessionId = login();
        
        //找出任务正在执行的所有实例
        Integer[] execIds = getFlowRuning(sessionId, projectName, flowName);
        String execIdsStr = "";
        //将所有的实例停止
        for (Integer execid : execIds) {
        	LOGGER.info("######执行实例execid:"+execid);
        	cancleFlow(sessionId, execid);
        	execIdsStr= execIdsStr +","+ execid;
		}

        return execIdsStr;
    }
    
    /**
     * 获取任务所有正在执行的实例
     *
     * @param sessionId   会话id
     * @param projectName 项目名
     * @param flowName    azkaban工作流名称
     * @return 调用成功后，返回execid
     */
    @SuppressWarnings("unchecked")
    public Integer[] getFlowRuning(String sessionId, String projectName, String flowName) {
        if (StringUtils.isEmpty(sessionId)) {
            throw new RuntimeException("还未登陆");
        }

        if (StringUtils.isEmpty(projectName)) {
            throw new RuntimeException("调度流程项目名不能为空");
        }

        if (StringUtils.isEmpty(flowName)) {
            throw new RuntimeException("调度流程  流程名不能为空");
        }

        String executeFlowUrl = azkabanServerUrl + "/executor?token=" + sessionId;
        LinkedMultiValueMap<String, Object> linkedMultiValueMap = new LinkedMultiValueMap<String, Object>();
        linkedMultiValueMap.add("session.id", sessionId);
        linkedMultiValueMap.add("ajax", "getRunning");
        linkedMultiValueMap.add("project", projectName);
        linkedMultiValueMap.add("flow", flowName);

        Map<String, Object> postForObject = null;
        try {
            postForObject = bulidRestTemplate().postForObject(executeFlowUrl, linkedMultiValueMap, Map.class);
        } catch (Exception e) {
            LOGGER.error(executeFlowUrl + "----" + linkedMultiValueMap + "----获取任务正在调度实例接口异常");
            e.printStackTrace();
            throw new RuntimeException("获取任务调度实例接口异常");
        }

        if (postForObject.containsKey("error")){
            throw new RuntimeException(postForObject.get("error").toString());
        }
        String status = (String) postForObject.get("status");
        if ("error".equals(status)) {
            String message = (String) postForObject.get("message");
            throw new RuntimeException(message);
        }
        
        Integer[] execIds = new Integer[] {};
        
        if (postForObject.get("execIds")!=null) {
        	execIds = (Integer[])postForObject.get("execIds");
		}
        
        LOGGER.info("获取所有任务("+projectName+")正在调度execids"+execIds.toString());
        
        return execIds;
    }
    
    /**
     * 获取任务所有正在执行的实例
     *
     * @param sessionId   会话id
     * @param projectName 项目名
     * @param flowName    azkaban工作流名称
     * @return 调用成功后，返回execid
     */
    @SuppressWarnings("unchecked")
    public boolean cancleFlow(String sessionId, Integer execid) {
        if (StringUtils.isEmpty(sessionId)) {
            throw new RuntimeException("还未登陆");
        }

        String executeFlowUrl = azkabanServerUrl + "/executor?token=" + sessionId;
        LinkedMultiValueMap<String, Object> linkedMultiValueMap = new LinkedMultiValueMap<String, Object>();
        linkedMultiValueMap.add("session.id", sessionId);
        linkedMultiValueMap.add("ajax", "cancelFlow");
        linkedMultiValueMap.add("execid", execid);

        Map<String, Object> postForObject = null;
        try {
            postForObject = bulidRestTemplate().postForObject(executeFlowUrl, linkedMultiValueMap, Map.class);
        } catch (Exception e) {
            LOGGER.error(executeFlowUrl + "----" + linkedMultiValueMap + "----停止任务实例（"+execid+"）调度接口异常");
            e.printStackTrace();
            throw new RuntimeException("亭子任务调度实例（"+execid+"）接口异常");
        }

        if (postForObject.containsKey("error")){
            throw new RuntimeException(postForObject.get("error").toString());
        }
        String status = (String) postForObject.get("status");
        if ("error".equals(status)) {
            String message = (String) postForObject.get("message");
            throw new RuntimeException(message);
        }
        
        LOGGER.info("停止调度（"+execid+"）成功");
        
        return true;
    }

    public static void main(String[] args) throws Exception {
        AzkabanApiUtils2 azkabanApiUtils = new AzkabanApiUtils2("http://119.23.32.151:8083");
        boolean dw_test = azkabanApiUtils.checkFlowExists("dw_test", "123");
        System.err.println(dw_test);
    }

	/**  
	* @Title: getSyncingFlowExecution  
	* @Description: TODO(获取同步状态结果)  
	* @param @param projectId
	* @param @param flow
	* @param @param start
	* @param @param length
	* @param @return    参数  
	* @return List<FlowExecution>    返回类型  
	* @throws  
	*/ 
	public List<FlowExecution> getSyncingFlowExecution(Long projectId, String flow, Integer start, Integer length) {

		List<FlowExecution> list = new ArrayList<FlowExecution>();

		//登录
        String sessionId = login();
        //String sessionId = "2c2e0af8-e5d4-46be-af55-f708f4447630";

        String executeFlowUrl = azkabanServerUrl + "/manager?token=" + sessionId;
        Map<String, Object> paramMap = new HashMap<String, Object>();
        paramMap.put("session.id", sessionId);
        paramMap.put("ajax", "fetchFlowExecutions");
        paramMap.put("project", "jz_localflow_"+projectId);
        paramMap.put("flow", flow);
        paramMap.put("start", start);
        paramMap.put("length", length);

        JSONObject getForObject = null;
        try {
            //postForObject = bulidRestTemplate().postForObject(executeFlowUrl, linkedMultiValueMap, Map.class);
            //String getForObjectStr = bulidRestTemplate().getForObject(executeFlowUrl, String.class, paramMap);
        	String getForObjectStr = HttpClientUtils.getJsonForParam(executeFlowUrl, paramMap);
            getForObject = JSON.parseObject(getForObjectStr);
        } catch (Exception e) {
            LOGGER.error(executeFlowUrl + "----" + paramMap + "----获取同步状态结果接口异常");
            e.printStackTrace();
            throw new RuntimeException("获取同步状态结果接口异常");
        }

        if (getForObject==null) {
			return list;
		}

        if (getForObject.containsKey("error")){
            throw new RuntimeException(getForObject.get("error").toString());
        }
        String status = (String) getForObject.get("status");
        if ("error".equals(status)) {
            String message = (String) getForObject.get("message");
            throw new RuntimeException(message);
        }

        String executions = getForObject.getString("executions");

        LOGGER.info("获取同步状态结果成功："+executions);

        list = JSONObject.parseArray(executions, FlowExecution.class);

		return list;
	}
}

