package com.jz.dmp.modules.controller;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.bean.BaseResponse;
import com.jz.common.bean.PageInfoResponse;
import com.jz.common.constant.StatuConstant;
import com.jz.dmp.modules.controller.bean.DmpDevelopTaskDto;
import com.jz.dmp.modules.controller.bean.DmpDevelopTaskRequest;
import com.jz.dmp.modules.model.DmpDevelopTask;
import com.jz.dmp.modules.service.DmpDevelopTaskService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 任务开发(DmpDevelopTask)表控制层
 *
 * @author Bellamy
 * @since 2020-12-22 16:37:56
 */
@RestController
@RequestMapping("/dmpDevelopTask")
@Api(tags = "任务开发")
public class DmpDevelopTaskController {
    /**
     * 服务对象
     */
    @Autowired
    private DmpDevelopTaskService dmpDevelopTaskService;
    
    /**列表查询任务开发
	 * @param dmpDevelopTaskRequest
	 * @return
	 */
	@RequestMapping(method = RequestMethod.POST, value = "/findListWithPage")
	@ApiOperation(value = "分頁列表查询任务开发", notes = "分頁列表查询任务开发")
	public PageInfoResponse<DmpDevelopTaskDto> findListWithPage(@RequestBody DmpDevelopTaskRequest dmpDevelopTaskRequest, HttpServletRequest httpRequest){
		PageInfoResponse<DmpDevelopTaskDto> pageInfo = new PageInfoResponse<DmpDevelopTaskDto>();
		try {
			pageInfo = dmpDevelopTaskService.findListWithPage(dmpDevelopTaskRequest, httpRequest);
		} catch (Exception e) {
			pageInfo.setMessage("查询失败");
			pageInfo.setCode(StatuConstant.FAILURE_CODE);
			e.printStackTrace();
		}
		
		return pageInfo;
	}
	
	/**新增开发任务
	 * @param dmpDevelopTaskRequest
	 * @return
	 */
	@RequestMapping(method = RequestMethod.POST, value = "/add")
	@ApiOperation(value = "新增开发任务", notes = "新增开发任务")
	public BaseBeanResponse<DmpDevelopTask> add(@RequestBody DmpDevelopTask dmpDevelopTask, HttpServletRequest httpRequest){
		BaseBeanResponse<DmpDevelopTask> baseBeanResponse = new BaseBeanResponse<DmpDevelopTask>();
		try {
			baseBeanResponse = dmpDevelopTaskService.add(dmpDevelopTask, httpRequest);
		} catch (Exception e) {
			baseBeanResponse.setMessage("新增或修改开发任务失败");
			baseBeanResponse.setCode(StatuConstant.FAILURE_CODE);
			e.printStackTrace();
		}
		
		return baseBeanResponse;
	}
	
	/**任务流程保存提交
	 * @param dmpDevelopTaskRequest
	 * @return
	 */
	@RequestMapping(method = RequestMethod.POST, value = "/flowSubmit")
	@ApiOperation(value = "任务流程保存提交", notes = "任务流程保存提交")
	public BaseResponse flowSubmit(@RequestBody DmpDevelopTask dmpDevelopTask, HttpServletRequest httpRequest){
		BaseResponse baseResponse = new BaseResponse();
		
		try {
			baseResponse = dmpDevelopTaskService.flowSubmit(dmpDevelopTask, httpRequest);
		} catch (Exception e) {
			baseResponse.setMessage("任务流程保存提交失败");
			baseResponse.setCode(StatuConstant.FAILURE_CODE);
			e.printStackTrace();
		}
		
		return baseResponse;
	}

}