package com.jz.dmp.modules.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.common.utils.DateUtils;
import com.jz.common.utils.web.HttpClientUtils;
import com.jz.common.utils.web.SessionUtils;
import com.jz.dmp.modules.controller.DataIntegration.bean.SourceDbNameListDto;
import com.jz.dmp.modules.controller.dataService.bean.ApiInterfaceInfoListReq;
import com.jz.dmp.modules.controller.dataService.bean.ApiInterfaceReq;
import com.jz.dmp.modules.controller.dataService.bean.MakeBigDataApiReq;
import com.jz.dmp.modules.dao.DvRuleTDao;
import com.jz.dmp.modules.dao.OfflineSynchDao;
import com.jz.dmp.modules.model.DvRuleT;
import com.jz.dmp.modules.service.DmpApiServiceMangeService;
import com.jz.dmp.modules.service.DvRuleTService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @ClassName: DmpApiServiceMangeServiceImpl
 * @Description: 数据服务-服务管理--调用gateway 接口
 * @Author: Bellamy
 * @Date 2021/1/19
 * @Version 1.0
 */
@Service("dmpApiServiceMangeService")
public class DmpApiServiceMangeServiceImpl implements DmpApiServiceMangeService {

    private static Logger logger = LoggerFactory.getLogger(DmpApiServiceMangeServiceImpl.class);

    //保存API(第三方)基本信息url
    private static final String addApiInfo = "/api/producer/addCustomApi";

    //保存API(大数据查询/标签查询)url
    private static final String addApiBigData = "/api/producer/makeBigDataApi";

    //编辑API(第三方)基本信息url
    private static final String updateApiInfo = "/api/producer/saveUpdate";

    //API计量调用次数和执行时长url
    private static final String countAPiCallStat = "/api/logging/countAPiCallStat";

    //API计量--API未调用列表url
    private static final String notCalledList = "/api/interface/listUnCallApiInterface";

    @Value("${spring.gateway-url}")
    private String gatewayUrl;

    @Value("${spring.api-bigData-setting}")
    private String apiBigSetting;

    @Autowired
    private OfflineSynchDao offlineSynchDao;

    /**
     * 保存API(第三方)基本信息
     *
     * @author Bellamy
     * @since 2021-01-19
     */
    @Override
    public JsonResult saveApiInfo(ApiInterfaceReq req) throws Exception {
        JsonResult result = new JsonResult();
        String url = gatewayUrl + addApiInfo;

        req.setCreateUser(SessionUtils.getCurrentUserName());
        String resultData = HttpClientUtils.post(url, JSONObject.toJSONString(req));
        if (StringUtils.isEmpty(resultData)) {
            throw new RuntimeException("保存失败！");
        }
        logger.info("#################响应结果数据{}" + resultData);

        Map jsonObject = JSONObject.parseObject(resultData);
        if (jsonObject.containsKey("code")) {
            if ("200".equals(jsonObject.get("code").toString())) {
                Map apiKey = JSONObject.parseObject(jsonObject.get("data").toString());
                result.setData(apiKey.get("data")); // 新增API数据的apiKey
                return result;
            }
        }
        if (jsonObject.containsKey("message")) {
            logger.info(jsonObject.get("message").toString());
            result.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            result.setMessage(jsonObject.get("message").toString());
        }
        return result;
    }

    /**
     * 保存API(大数据查询/标签查询)
     *
     * @author Bellamy
     * @since 2021-01-19
     */
    @Override
    public JsonResult saveApiBigDataInfo(MakeBigDataApiReq req) throws Exception {
        JsonResult result = new JsonResult();
        ApiInterfaceReq baseInfo = new ApiInterfaceReq();
        BeanUtils.copyProperties(req, baseInfo);
        //保存API基本信息
        JsonResult apiKeyData = saveApiInfo(baseInfo);
        String apiKey = apiKeyData.getData().toString();
        if (StringUtils.isNotEmpty(apiKey)) {
            logger.info("######apiKey=" + apiKey);

            req.setApiKey(apiKey);
            req.setCreateUser(SessionUtils.getCurrentUserName());
            String url = gatewayUrl + addApiBigData;//保存API(大数据查询/标签查询)
            String resultData = HttpClientUtils.post(url, JSONObject.toJSONString(req));
            if (StringUtils.isEmpty(resultData)) {
                throw new RuntimeException("保存失败！");
            }
            logger.info("#################响应结果数据{}" + resultData);

            Map jsonObject = JSONObject.parseObject(resultData);
            if (jsonObject.containsKey("code")) {
                if ("200".equals(jsonObject.get("code").toString())) {
                    return result;
                }
            }
            if (jsonObject.containsKey("message")) {
                logger.info(jsonObject.get("message").toString());
                result.setCode(ResultCode.INTERNAL_SERVER_ERROR);
                result.setMessage(jsonObject.get("message").toString());
            }
        }
        return result;
    }

    /**
     * 编辑API(第三方)基本信息
     *
     * @author Bellamy
     * @since 2021-01-20
     */
    @Override
    public JsonResult updateApiInfo(ApiInterfaceReq req) throws Exception {
        JsonResult result = new JsonResult();
        String url = gatewayUrl + updateApiInfo;

        req.setCreateUser(SessionUtils.getCurrentUserName());
        String resultData = HttpClientUtils.post(url, JSONObject.toJSONString(req));
        if (StringUtils.isEmpty(resultData)) {
            throw new RuntimeException("编辑失败！");
        }
        logger.info("#################响应结果数据{}" + resultData);

        Map jsonObject = JSONObject.parseObject(resultData);
        if (jsonObject.containsKey("code")) {
            if ("200".equals(jsonObject.get("code").toString())) {
                return result;
            }
        }
        if (jsonObject.containsKey("message")) {
            logger.info(jsonObject.get("message").toString());
            result.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            result.setMessage(jsonObject.get("message").toString());
        }
        return result;
    }

    /**
     * 编辑API(大数据查询/标签查询)
     *
     * @author Bellamy
     * @since 2021-01-20
     */
    @Override
    public JsonResult updateApiBigDataInfo(MakeBigDataApiReq req) throws Exception {
        ApiInterfaceReq baseApi = new ApiInterfaceReq();
        BeanUtils.copyProperties(req, baseApi);
        //编辑API基本信息
        JsonResult baseRes = updateApiInfo(baseApi);
        if (!"200".equals(baseRes.getCode())) {
            throw new RuntimeException(baseRes.getMessage());
        }

        //编辑API(大数据查询/标签查询)
        JsonResult result = new JsonResult();
        req.setCreateUser(SessionUtils.getCurrentUserName());
        String url = gatewayUrl + addApiBigData;
        String resultData = HttpClientUtils.post(url, JSONObject.toJSONString(req));
        if (StringUtils.isEmpty(resultData)) {
            throw new RuntimeException("编辑失败！");
        }
        logger.info("#################响应结果数据{}" + resultData);

        Map jsonObject = JSONObject.parseObject(resultData);
        if (jsonObject.containsKey("code")) {
            if ("200".equals(jsonObject.get("code").toString())) {
                return result;
            }
        }
        if (jsonObject.containsKey("message")) {
            logger.info(jsonObject.get("message").toString());
            result.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            result.setMessage(jsonObject.get("message").toString());
        }
        return result;
    }

    /**
     * API计量调用次数和执行时长
     *
     * @author Bellamy
     * @since 2021-01-20
     */
    @Override
    public JsonResult getCountAPiCallStat() throws Exception {
        JsonResult result = new JsonResult();
        String url = gatewayUrl + countAPiCallStat;

        Map params = new HashMap();
        params.put("date", DateUtils.currentDate());
        String returnData = HttpClientUtils.getJsonForParam(url, params);
        if (StringUtils.isEmpty(returnData)) {
            throw new RuntimeException("查询失败！");
        }
        logger.info("#################响应结果{}" + returnData);
        Map map = JSONObject.parseObject(returnData);
        if (map.containsKey("code")) {
            if ("200".equals(map.get("code").toString())) {
                return JsonResult.ok(map.get("data"));
            }
        }
        if (map.containsKey("message")) {
            logger.info(map.get("message").toString());
            result.setMessage(map.get("message").toString());
            result.setCode(ResultCode.INTERNAL_SERVER_ERROR);
        }
        return result;
    }

    /**
     * 获取数据源库——下拉框
     *
     * @return
     * @author Bellamy
     */
    @Override
    public JsonResult querygSourceDbList(Integer projectId,String dbName) throws Exception {
        Map map = new HashMap();
        map.put("projectId", projectId); //项目id
        map.put("dbName", dbName);
        List<SourceDbNameListDto> list = offlineSynchDao.querygSourceDbList(map);
        return JsonResult.ok(list);
    }

    /**
     * 配置数据源——下拉框
     *
     * @return
     * @author Bellamy
     * @since 2021-01-21
     */
    @Override
    public JsonResult getSourceSetting() throws Exception {
        String[] arr = apiBigSetting.split(",");
        List<String> list = Arrays.asList(arr);
        return JsonResult.ok(list);
    }

    /**
     * API计量--API未调用列表
     *
     * @author Bellamy
     * @since 2021-01-22
     */
    @Override
    public JsonResult queryApiNotCalledListPage(ApiInterfaceInfoListReq req) throws Exception {
        JsonResult result = new JsonResult();
        String url = gatewayUrl + notCalledList;
        String resultData = HttpClientUtils.post(url, JSONObject.toJSONString(req));
        if (StringUtils.isEmpty(resultData)) {
            throw new RuntimeException("查询失败！");
        }
        logger.info("#################响应结果数据{}" + resultData);

        Map jsonObject = JSONObject.parseObject(resultData);
        if (jsonObject.containsKey("code")) {
            if ("200".equals(jsonObject.get("code").toString())) {
                return JsonResult.ok(jsonObject.get("data"));
            }
        }
        if (jsonObject.containsKey("message")) {
            logger.info(jsonObject.get("message").toString());
            result.setMessage(jsonObject.get("message").toString());
            result.setCode(ResultCode.INTERNAL_SERVER_ERROR);
        }
        return result;
    }
}