package com.jz.dmp.modules.service.impl;

import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.bean.BaseResponse;
import com.jz.common.constant.StatuConstant;
import com.jz.common.utils.FlowParseTool;
import com.jz.common.utils.GZIPUtils;
import com.jz.dmp.modules.controller.DataIntegration.bean.flow.FlowNodeChangeInfo;
import com.jz.dmp.modules.controller.DataIntegration.bean.flow.FlowPro;
import com.jz.dmp.modules.controller.projconfig.bean.DmpProjectConfigInfoDto;
import com.jz.dmp.modules.dao.DmpProjectDao;
import com.jz.dmp.modules.dao.DmpWorkFlowSubmitDetailsDao;
import com.jz.dmp.modules.dao.projconfig.DmpProjectConfigInfoMapper;
import com.jz.dmp.modules.model.DmpProject;
import com.jz.dmp.modules.model.DmpProjectSystemInfo;
import com.jz.dmp.modules.model.DmpWorkFlowSubmitDetails;
import com.jz.dmp.modules.service.DmpDevelopTaskService;
import com.jz.dmp.modules.service.DmpNavigationTreeService;
import com.jz.dmp.modules.service.DmpProjectService;
import com.jz.dmp.modules.service.DmpWorkFlowSubmitDetailsService;
import com.jz.dmp.modules.service.FlowService;
import com.jz.dmp.modules.service.projconfig.DmpProjectConfigInfoService;

/**
 * @ClassName: FlowServiceImpl
 * @Description: TODO(工作流任务)
 * @author ybz
 * @date 2021年1月19日
 *
 */
@Service
public class FlowServiceImpl implements FlowService {
	@Autowired
	private DmpProjectDao dmpProjectDao;
	
	@Autowired
	private DmpProjectService dmpProjectService;
	
    @Autowired
    private DmpDevelopTaskService dmpDevelopTaskService;

    @Autowired
    private DmpNavigationTreeService dmpNavigationTreeService;
    
    @Autowired
    private DmpWorkFlowSubmitDetailsService dmpWorkFlowSubmitDetailsService;
    
    @Autowired
    private DmpWorkFlowSubmitDetailsDao dmpWorkFlowSubmitDetailsDao;
    
    @Autowired
    private DmpProjectConfigInfoService dmpProjectConfigInfoService;

	/**
	 *工作流发布
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseResponse publishFlow(FlowPro flowPro) throws Exception {
		BaseResponse baseResponse = new BaseResponse();
		
		flowPro.setCheckVerion(true);
        Long publishedToProjectId = flowPro.getPublishedToProjectId();
        DmpProject publishToProject = dmpProjectDao.get(publishedToProjectId);
        //DmpProjectSystemInfo publishToProjectSystemInfo = dmpProjectService.getProjectSystemInfo(publishedToProjectId);
        BaseBeanResponse<DmpProjectConfigInfoDto> baseBeanResponse = dmpProjectConfigInfoService.findByProjectId(publishedToProjectId.intValue(), null);
        DmpProjectConfigInfoDto dmpProjectConfigInfoDto = baseBeanResponse.getData();
        
        try {
        	/*
            FlowParseTool flowParseTool = new FlowParseTool(flowPro,
                    publishToProject,
                    publishToProjectSystemInfo,
                    dmpDevelopTaskService,
                    dmpNavigationTreeService,
                    dmpWorkFlowSubmitDetailsService
            );
            */
        	
        	FlowParseTool flowParseTool = new FlowParseTool(flowPro,
                    publishToProject,
                    dmpProjectConfigInfoDto,
                    dmpDevelopTaskService,
                    dmpNavigationTreeService,
                    dmpWorkFlowSubmitDetailsService
            );

            //保存发布信息
            List<FlowNodeChangeInfo> flowNodeChangeList = flowParseTool.getChangedNodes();
            if (flowNodeChangeList == null || flowNodeChangeList.size() == 0) {
            	baseResponse.setCode(StatuConstant.CODE_DATA_NOTMEET);
            	baseResponse.setMessage("流程节点数据没有变化，取消发布");
            	return baseResponse;
            }

            boolean successed = flowParseTool.publish();
            if (!successed) {
            	baseResponse.setCode(StatuConstant.FAILURE_CODE);
            	baseResponse.setMessage("发布失败");
            	return baseResponse;
            }

            if (flowNodeChangeList != null && flowNodeChangeList.size() >= 1) {
                DmpWorkFlowSubmitDetails dmpWorkFlowSubmitDetails = null;
                FlowNodeChangeInfo flowNodeChangeInfo = null;
                int size = flowNodeChangeList.size();
                for (int i = 0; i < size; i++) {
                    flowNodeChangeInfo = flowNodeChangeList.get(i);
                    dmpWorkFlowSubmitDetails = new DmpWorkFlowSubmitDetails();
                    dmpWorkFlowSubmitDetails.setScheduleProjectId(publishedToProjectId);
                    dmpWorkFlowSubmitDetails.setScheduleFlowName(flowParseTool.getFlowPro().getFlowName());
                    dmpWorkFlowSubmitDetails.setNodeName(flowNodeChangeInfo.getNodeName());
                    dmpWorkFlowSubmitDetails.setNodeDataBytes(GZIPUtils.gzipString(flowNodeChangeInfo.getNodeData(), "utf-8"));
                    dmpWorkFlowSubmitDetails.setVersion(flowNodeChangeInfo.getVersion());
                    dmpWorkFlowSubmitDetails.setCreatedTime(new Date());
                    dmpWorkFlowSubmitDetails.setCreatedBy("superadmin");
                    dmpWorkFlowSubmitDetails.setIsGziped(1);
                    dmpWorkFlowSubmitDetailsDao.insert(dmpWorkFlowSubmitDetails);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
        
        baseResponse.setCode(StatuConstant.SUCCESS_CODE);
        baseResponse.setCode("发布成功");

        return baseResponse;
	}

}
