package com.jz.dmp.modules.service.sys.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.bean.BaseResponse;
import com.jz.common.bean.PageInfoResponse;
import com.jz.common.constant.StatuConstant;
import com.jz.common.persistence.BaseService;
import com.jz.dmp.modules.controller.sys.bean.DmpMemberProjectRoleBatch;
import com.jz.dmp.modules.controller.sys.bean.DmpMemberProjectRoleDto;
import com.jz.dmp.modules.controller.sys.bean.DmpMemberProjectRoleRequest;
import com.jz.dmp.modules.controller.sys.bean.MembProjRoleBatchReq;
import com.jz.dmp.modules.dao.sys.DmpMemberProjectRoleMapper;
import com.jz.dmp.modules.model.DmpMemberProjectRole;
import com.jz.dmp.modules.service.sys.DmpMemberProjectRoleService;

/**
 * 用户项目角色表服务的实现?
 * @author ybz
 *
 */
@Service("dmpMemberProjectRoleService")
public class DmpMemberProjectRoleServiceImpl extends BaseService implements DmpMemberProjectRoleService {
	
	private static Logger logger = LoggerFactory.getLogger(DmpMemberProjectRoleServiceImpl.class);
	
	@Autowired
	private DmpMemberProjectRoleMapper dmpMemberProjectRoleMapper;

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpMemberProjectRoleService#findList(com.ycxc.vmts.controller.bean.DmpMemberProjectRoleRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public PageInfoResponse<DmpMemberProjectRoleDto> findListWithPage(DmpMemberProjectRoleRequest dmpMemberProjectRoleRequest, HttpServletRequest httpRequest)
			throws Exception {
		PageInfoResponse<DmpMemberProjectRoleDto> pageInfoResponse = new PageInfoResponse<>();
		
		Map<String, Object> param = new HashMap<>();
		//主键
		if (dmpMemberProjectRoleRequest.getMemberProjectRoleId() != null) {
			param.put("memberProjectRoleId", dmpMemberProjectRoleRequest.getMemberProjectRoleId());
		}
		//用户ID
		if (dmpMemberProjectRoleRequest.getUserId() != null) {
			param.put("userId", dmpMemberProjectRoleRequest.getUserId());
		}
		//项目ID
		if (dmpMemberProjectRoleRequest.getProjectId() != null) {
			param.put("projectId", dmpMemberProjectRoleRequest.getProjectId());
		}
		//角色ID
		if (dmpMemberProjectRoleRequest.getRoleId() != null) {
			param.put("roleId", dmpMemberProjectRoleRequest.getRoleId());
		}
		
		PageHelper.startPage(dmpMemberProjectRoleRequest.getPageNum(), dmpMemberProjectRoleRequest.getPageSize());
		List<DmpMemberProjectRoleDto> list = dmpMemberProjectRoleMapper.findList(param);
		PageInfo<DmpMemberProjectRoleDto> pageInfo = new PageInfo<>(list);
		
		pageInfoResponse.setCode(StatuConstant.SUCCESS_CODE);
		pageInfoResponse.setMessage("查询成功");
		pageInfoResponse.setData(pageInfo);
		return pageInfoResponse;
	}
	
	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpMemberProjectRoleService#findList(com.ycxc.vmts.controller.bean.DmpMemberProjectRoleRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public BaseBeanResponse<DmpMemberProjectRoleDto> findList(DmpMemberProjectRoleRequest dmpMemberProjectRoleRequest, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<DmpMemberProjectRoleDto> baseBeanResponse = new BaseBeanResponse<>();
		
		Map<String, Object> param = new HashMap<>();
		//主键
		if (dmpMemberProjectRoleRequest.getMemberProjectRoleId() != null) {
			param.put("memberProjectRoleId", dmpMemberProjectRoleRequest.getMemberProjectRoleId());
		}
		//用户ID
		if (dmpMemberProjectRoleRequest.getUserId() != null) {
			param.put("userId", dmpMemberProjectRoleRequest.getUserId());
		}
		//项目ID
		if (dmpMemberProjectRoleRequest.getProjectId() != null) {
			param.put("projectId", dmpMemberProjectRoleRequest.getProjectId());
		}
		//角色ID
		if (dmpMemberProjectRoleRequest.getRoleId() != null) {
			param.put("roleId", dmpMemberProjectRoleRequest.getRoleId());
		}
		
		List<DmpMemberProjectRoleDto> list = dmpMemberProjectRoleMapper.findList(param);
		
		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setDatas(list);
		return baseBeanResponse;
	}
	

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpMemberProjectRoleService#add(com.ycxc.vmts.entity.DmpMemberProjectRole, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseBeanResponse<DmpMemberProjectRole> add(DmpMemberProjectRole dmpMemberProjectRole, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpMemberProjectRole> baseBeanResponse = new BaseBeanResponse<>();
		
		/*$*setMemberProjectRoleIdCodeReplace*$*/
		//创建人
		dmpMemberProjectRole.setCreateUserId(getHttpRequestUserId(httpRequest));
		//创建时间
		dmpMemberProjectRole.setCreateTime(new Date());
		
		dmpMemberProjectRoleMapper.insertSelective(dmpMemberProjectRole);
		
		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("新增成功");
		baseBeanResponse.setData(dmpMemberProjectRole);
		
		return baseBeanResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpMemberProjectRoleService#edit(com.ycxc.vmts.entity.DmpMemberProjectRole, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseBeanResponse<DmpMemberProjectRole> edit(DmpMemberProjectRole dmpMemberProjectRole, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpMemberProjectRole> baseBeanResponse = new BaseBeanResponse<>();
		
		//修改人
		dmpMemberProjectRole.setUpdateUserId(getHttpRequestUserId(httpRequest));
		//修改时间
		dmpMemberProjectRole.setUpdateTime(new Date());
		
		dmpMemberProjectRoleMapper.updateByPrimaryKeySelective(dmpMemberProjectRole);
		
		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("修改成功");
		baseBeanResponse.setData(dmpMemberProjectRole);
		
		return baseBeanResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpMemberProjectRoleService#findById(com.ycxc.vmts.controller.bean.DmpMemberProjectRoleRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public BaseBeanResponse<DmpMemberProjectRoleDto> findById(Integer memberProjectRoleId, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<DmpMemberProjectRoleDto> baseBeanResponse = new BaseBeanResponse<>();
		
		DmpMemberProjectRoleDto dmpMemberProjectRoleDto = dmpMemberProjectRoleMapper.findById(memberProjectRoleId);
		
		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setData(dmpMemberProjectRoleDto);
		
		return baseBeanResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpMemberProjectRoleService#deleteById(com.ycxc.vmts.controller.bean.DmpMemberProjectRoleRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseResponse deleteById(Integer memberProjectRoleId, HttpServletRequest httpRequest)
			throws Exception {
		BaseResponse baseResponse = new BaseResponse();
		
		dmpMemberProjectRoleMapper.deleteByPrimaryKey(memberProjectRoleId);
		
		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("删除成功");
		
		return baseResponse;
	}
	
	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpMemberProjectRoleService#deleteById(com.ycxc.vmts.controller.bean.DmpMemberProjectRoleRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseResponse softDeleteById(Integer memberProjectRoleId, HttpServletRequest httpRequest)
			throws Exception {
		BaseResponse baseResponse = new BaseResponse();
		
		dmpMemberProjectRoleMapper.softDeleteByPrimaryKey(memberProjectRoleId);
		
		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("软删除成功");
		
		return baseResponse;
	}
	
	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpMemberProjectRoleService#deleteById(com.ycxc.vmts.controller.bean.DmpMemberProjectRoleRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseResponse delete(DmpMemberProjectRoleRequest dmpMemberProjectRoleRequest, HttpServletRequest httpRequest)
			throws Exception {
		BaseResponse baseResponse = new BaseResponse();
		
		Map<String, Object> param = new HashMap<>();
		//主键
		if (dmpMemberProjectRoleRequest.getMemberProjectRoleId() != null) {
			param.put("memberProjectRoleId", dmpMemberProjectRoleRequest.getMemberProjectRoleId());
		}
		//用户ID
		if (dmpMemberProjectRoleRequest.getUserId() != null) {
			param.put("userId", dmpMemberProjectRoleRequest.getUserId());
		}
		//项目ID
		if (dmpMemberProjectRoleRequest.getProjectId() != null) {
			param.put("projectId", dmpMemberProjectRoleRequest.getProjectId());
		}
		//角色ID
		if (dmpMemberProjectRoleRequest.getRoleId() != null) {
			param.put("roleId", dmpMemberProjectRoleRequest.getRoleId());
		}
		
		dmpMemberProjectRoleMapper.delete(param);
		
		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("删除成功");
		
		return baseResponse;
	}
	
	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpMemberProjectRoleService#deleteById(com.ycxc.vmts.controller.bean.DmpMemberProjectRoleRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseResponse softDelete(DmpMemberProjectRoleRequest dmpMemberProjectRoleRequest, HttpServletRequest httpRequest)
			throws Exception {
		BaseResponse baseResponse = new BaseResponse();
		
		Map<String, Object> param = new HashMap<>();
		//主键
		if (dmpMemberProjectRoleRequest.getMemberProjectRoleId() != null) {
			param.put("memberProjectRoleId", dmpMemberProjectRoleRequest.getMemberProjectRoleId());
		}
		//用户ID
		if (dmpMemberProjectRoleRequest.getUserId() != null) {
			param.put("userId", dmpMemberProjectRoleRequest.getUserId());
		}
		//项目ID
		if (dmpMemberProjectRoleRequest.getProjectId() != null) {
			param.put("projectId", dmpMemberProjectRoleRequest.getProjectId());
		}
		//角色ID
		if (dmpMemberProjectRoleRequest.getRoleId() != null) {
			param.put("roleId", dmpMemberProjectRoleRequest.getRoleId());
		}
		
		dmpMemberProjectRoleMapper.softDelete(param);
		
		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("软删除成功");
		
		return baseResponse;
	}

	/* (non-Javadoc)
	 * @see com.ycxc.vmts.service.DmpMemberProjectRoleService#addBatch(com.ycxc.vmts.controller.bean.DmpMemberProjectRoleVo, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseBeanResponse<DmpMemberProjectRoleBatch> addBatch(DmpMemberProjectRoleBatch dmpMemberProjectRoleBatch, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpMemberProjectRoleBatch> baseBeanResponse = new BaseBeanResponse<>();
		
		List<DmpMemberProjectRole> dmpMemberProjectRoles = dmpMemberProjectRoleBatch.getDmpMemberProjectRoles();
		
		if (CollectionUtils.isEmpty(dmpMemberProjectRoles)) {
			baseBeanResponse.setCode(StatuConstant.FAILURE_CODE);
			baseBeanResponse.setMessage("数据不能为空！");
			return baseBeanResponse;
		}
		
		dmpMemberProjectRoles = dmpMemberProjectRoles.stream().map(dmpMemberProjectRole -> {
			/*$*setMemberProjectRoleIdCodeReplace*$*/
			//创建人
			try {
				dmpMemberProjectRole.setCreateUserId(getHttpRequestUserId(httpRequest));
			} catch (Exception e) {
				logger.info(e.getMessage());
				e.printStackTrace();
			}
			//创建时间
			dmpMemberProjectRole.setCreateTime(new Date());
			
			return dmpMemberProjectRole;
		}).collect(Collectors.toList());
		
		dmpMemberProjectRoleMapper.insertBatch(dmpMemberProjectRoles);
		
		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("批量新增成功");
		baseBeanResponse.setData(dmpMemberProjectRoleBatch);
		
		return baseBeanResponse;
	}

	/**
	 *批量新增项目成员
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseBeanResponse<DmpMemberProjectRole> addProjMembers(MembProjRoleBatchReq membProjRoleBatchReq,
			HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpMemberProjectRole> baseBeanResponse = new BaseBeanResponse<DmpMemberProjectRole>();
		//判断用户是否为空
		if (CollectionUtils.isEmpty(membProjRoleBatchReq.getUserIds())) {
			baseBeanResponse.setCode(StatuConstant.CODE_ERROR_PARAMETER);
			baseBeanResponse.setMessage("用户ID不能为空！");
			return baseBeanResponse;
		}
		//判断项目ID是否为空
		if (StringUtils.isEmpty(membProjRoleBatchReq.getProjectId())) {
			baseBeanResponse.setCode(StatuConstant.CODE_ERROR_PARAMETER);
			baseBeanResponse.setMessage("项目不能为空！");
			return baseBeanResponse;
		}
		//判断角色是否为空
		if (CollectionUtils.isEmpty(membProjRoleBatchReq.getRoleIds())) {
			baseBeanResponse.setCode(StatuConstant.CODE_ERROR_PARAMETER);
			baseBeanResponse.setMessage("项目角色不能为空！");
			return baseBeanResponse;
		}
		
		List<Integer> userIds = membProjRoleBatchReq.getUserIds();
		List<Integer> roleIds = membProjRoleBatchReq.getRoleIds();
		Integer projectId = membProjRoleBatchReq.getProjectId();
		
		List<DmpMemberProjectRole> dmpMemberProjectRoles = new ArrayList<DmpMemberProjectRole>();
		
		for (Integer userId : userIds) {
			for (Integer roleId : roleIds) {
				DmpMemberProjectRole dmpMemberProjectRole = new DmpMemberProjectRole();
				dmpMemberProjectRole.setUserId(userId);
				dmpMemberProjectRole.setProjectId(projectId);
				dmpMemberProjectRole.setRoleId(roleId);
				dmpMemberProjectRoles.add(dmpMemberProjectRole);
			}
		}
		
		dmpMemberProjectRoles = dmpMemberProjectRoles.stream().map(dmpMemberProjectRole -> {
			/*$*setMemberProjectRoleIdCodeReplace*$*/
			//创建人
			try {
				dmpMemberProjectRole.setCreateUserId(getHttpRequestUserId(httpRequest));
			} catch (Exception e) {
				logger.info(e.getMessage());
				e.printStackTrace();
			}
			//创建时间
			dmpMemberProjectRole.setCreateTime(new Date());
			
			return dmpMemberProjectRole;
		}).collect(Collectors.toList());
		
		for (DmpMemberProjectRole dmpMemberProjectRole : dmpMemberProjectRoles) {
			dmpMemberProjectRoleMapper.insertSelective(dmpMemberProjectRole);
		}
		
		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("批量新增成功");
		return baseBeanResponse;
		
	}

}
